# -*- coding: utf-8 -*-
"""
Created on Mon Feb  1 16:42:45 2021

@author: kourf
"""
from graphviz import Graph, Digraph


class Graphed:
    "Graphe orienté non pondéré représenté par un dictionnaire"

    def __init__(self, sommets):
        self.sommets = {v: [] for v in sommets}
        self.n = n = len(sommets)

    def ajouter_arc(self, s1, s2):
        self.sommets[s1].append(s2)

    def get_sommets(self):
        return list(self.sommets.keys())

    def successeurs(self, s):
        return self.sommets[s]

    def voisins(self, s):
        "Identique à la méthode successeurs"
        return self.sommets[s]

    def est_oriente(self):
        for s1 in self.sommets:
            for s2 in self.sommets[s1]:
                if s1 not in self.sommets[s2]:
                    return True
        return False

    def montrer(self, oriente=None):
        if oriente==None:
            oriente = self.est_oriente()
        if oriente:
            g = Digraph('G', filename='graph.gv')
        else:
            g = Graph('G', filename='graph.gv')
        for s1 in self.sommets:
            for s2 in self.sommets[s1]:
                if oriente or s2 <= s1:
                    g.edge(s1, s2)
        g.view()
        return g


def graphe_objetd(g):
    "Renvoie un objet Graphed à partir d'un graphe sous forme de dictionnaire"


class Graphem:
    "Graphe orienté représenté par une matrice"

    def __init__(self, sommets):
        self.sommets = [s for s in sommets]
        self.n = n = len(sommets)
        self.arcs = [[0]*n for i in range(n)]

    def ajouter_arc(self, s1, s2, poids=1):
        i = self.sommets.index(s1)
        j = self.sommets.index(s2)
        self.arcs[i][j] = poids

    def get_sommets(self):
        return self.sommets

    def successeurs(self, s):
        i = self.sommets.index(s)
        return [self.sommets[j] for j in range(self.n)
                if self.arcs[i][j] != 0]

    def voisins(self, s):
        "Identique à la méthode successeurs"
        return self.successeurs(s)

    def est_successeur(self, s1, s2):
        i = self.sommets.index(s1)
        j = self.sommets.index(s2)
        return self.arcs[i][j] != 0

    def ajouter_sommet(self, s):
        # utiliser plutôt un dictionnaire en cas
        # d'ajouts/suppressions de sommets
        self.sommets.append(s)
        for i in range(self.n):
            self.arcs[i].append(0)
        self.n += 1
        self.arcs.append([0]*self.n)

    def supprimer_sommet(self, s):
        j = self.sommets.index(s)
        del self.sommets[j]
        del self.arcs[j]
        self.n -= 1
        for i in range(self.n):
            del self.arcs[i][j]

    def retourner_valeur_arc(self, s1, s2):
        i = self.sommets.index(s1)
        j = self.sommets.index(s2)
        return self.arcs[i][j]

    def est_oriente(self):
        for i in range(1, len(self.arcs)):
            for j in range(0, i):
                if self.arcs[i][j] != self.arcs[j][i]:
                    return True
        return False

    def montrer(self, oriente=None):
        if oriente==None:
            oriente = self.est_oriente()
        if oriente:
            g = Digraph('G', filename='graph.gv')
        else:
            g = Graph('G', filename='graph.gv')
        for i in range(self.n):
            for j in range(self.n):
                if self.arcs[i][j] != 0 and (oriente or i <= j):
                    if self.arcs[i][j] == 1:
                        g.edge(self.sommets[i], self.sommets[j])
                    else:
                        g.edge(self.sommets[i], self.sommets[j], label=str(self.arcs[i][j]))
        g.view()
        return g


def graphe_objetm(sommets, matrice):
    go = Graphem(sommets)
    n = len(sommets)
    for i in range(n):
        for j in range(n):
            if matrice[i][j]:
                go.arcs[i][j] = matrice[i][j]  # on se permet une petite incartade au principe d'encapsulation
    return go

def dico2matrice(gd):
    sommets = gd.get_sommets()
    gm = Graphem(sommets)
    for s1 in sommets:
        for s2 in gd.successeurs(s1):
            gm.ajouter_arc(s1, s2)
    return gm

def matrice2dico(gm):
    sommets = gm.get_sommets()
    gd = Graphed(sommets)
    for s1 in sommets:
        for s2 in gm.successeurs(s1):
            gd.ajouter_arc(s1, s2)
    return gd







